@@ -363,40 +363,50 @@ impl ServiceRunning {
     }
 }
 
+#[derive(Debug, thiserror::Error)]
+pub enum SystemdError<S: System> {
+    #[error("unable to execute systemctl: {0}")]
+    FailedToStart(S::CommandError),
+
+    #[error("systemctl failed: {0}")]
+    Unsuccessful(String),
+}
+
+#[derive(Debug, thiserror::Error)]
+#[error("unable to execute apt-get: {0}")]
+pub struct CheckError<S: System>(S::CommandError);
+
 impl Requirement for ServiceRunning {
-    type CreateError<S: System> = NeverError;
-    type ModifyError<S: System> = NeverError;
-    type DeleteError<S: System> = NeverError;
-    type HasBeenCreatedError<S: System> = NeverError;
+    type CreateError<S: System> = SystemdError<S>;
+    type ModifyError<S: System> = SystemdError<S>;
+    type DeleteError<S: System> = SystemdError<S>;
+    type HasBeenCreatedError<S: System> = CheckError<S>;
 
     fn create<S: crate::system::System>(&self, system: &mut S) -> Result<(), Self::CreateError<S>> {
         println!("  start: {}", self.name);
         let result = system
             .execute_command("systemctl", &["start", &self.name])
-            .unwrap();
-        assert!(result.is_success());
-
-        Ok(())
+            .map_err(SystemdError::FailedToStart)?;
+        
+        result.successful().map_err(|(stdout, stderr)| SystemdError::Unsuccessful(format!("{stdout}\n{stderr}")))
     }
 
     fn modify<S: crate::system::System>(&self, system: &mut S) -> Result<(), Self::ModifyError<S>> {
         println!("  restart: {}", self.name);
         let result = system
             .execute_command("systemctl", &["restart", &self.name])
-            .unwrap();
-        assert!(result.is_success());
-
-        Ok(())
+            .map_err(SystemdError::FailedToStart)?;
+        
+        result.successful().map_err(|(stdout, stderr)| SystemdError::Unsuccessful(format!("{stdout}\n{stderr}")))
     }
 
     fn delete<S: crate::system::System>(&self, system: &mut S) -> Result<(), Self::DeleteError<S>> {
         println!("  stop: {}", self.name);
         let result = system
             .execute_command("systemctl", &["stop", &self.name])
-            .unwrap();
-        assert!(result.is_success());
-
-        Ok(())
+            .map_err(SystemdError::FailedToStart)?;
+        
+        result.successful().map_err(|(stdout, stderr)| SystemdError::Unsuccessful(format!("{stdout}\n{stderr}")))
     }
 
     fn pre_existing_delete<S: crate::system::System>(
@@ -412,7 +422,7 @@ impl Requirement for ServiceRunning {
     ) -> Result<bool, Self::HasBeenCreatedError<S>> {
         let result = system
             .execute_command("systemctl", &["is-active", &self.name])
-            .unwrap();
+            .map_err(CheckError)?;
         Ok(result.is_success())
     }
 
@@ -456,12 +466,13 @@ impl InstallServices {
         context.add_node(InstallServices, dependencies)
     }
 
-    fn exec<S: System>(&self, system: &mut S) -> Result<(), NeverError> {
+    fn exec<S: System>(&self, system: &mut S) -> Result<(), SystemdError<S>> {
         println!("  service-reload");
         let result = system
             .execute_command("systemctl", &["daemon-reload"])
-            .unwrap();
-        assert!(result.is_success());
+            .map_err(SystemdError::FailedToStart)?;
+        // TODO: Better error
+        result.successful().map_err(|(stdout, stderr)| SystemdError::Unsuccessful(format!("{stdout}\n{stderr}")))?;
 
         let result = system
             .execute_command(
@@ -475,8 +486,7 @@ impl InstallServices {
                     "--no-pager",
                     "--full",
                 ],
-            )
-            .unwrap();
+            ).map_err(SystemdError::FailedToStart)?;
         for line in BufReader::new(&mut Cursor::new(&result.stdout_as_str())).lines() {
             let line = line.unwrap();
             let mut i = line.split_whitespace();
@@ -489,8 +499,10 @@ impl InstallServices {
                 println!("  not-found: {}", name);
                 let result = system
                     .execute_command("systemctl", &["stop", &name])
-                    .unwrap();
-                assert!(result.is_success());
+                    .map_err(SystemdError::FailedToStart)?;
+            
+                // TODO: Better error
+                result.successful().map_err(|(stdout, stderr)| SystemdError::Unsuccessful(format!("{stdout}\n{stderr}")))?;
             }
         }
 
@@ -499,9 +511,9 @@ impl InstallServices {
 }
 
 impl Requirement for InstallServices {
-    type CreateError<S: System> = NeverError;
-    type ModifyError<S: System> = NeverError;
-    type DeleteError<S: System> = NeverError;
+    type CreateError<S: System> = SystemdError<S>;
+    type ModifyError<S: System> = SystemdError<S>;
+    type DeleteError<S: System> = SystemdError<S>;
     type HasBeenCreatedError<S: System> = NeverError;
 
     fn create<S: crate::system::System>(&self, system: &mut S) -> Result<(), Self::CreateError<S>> {
